%% Navigation function path-planner
%% Author:  Jakob Stensvik Jensen
%% Date:    4/6-2020
function wp = nav_fun(Lx, Ly, Nx, Ny, r_TS, p_TS, vel_TS, r_HO, n_HO,...
    r_GW, kappa, lambda_psi, d_0, eta_0, wp, count)

persistent init_flag idx_old theta_old scenario

%% Configuration
% Partition parameters
dX = Lx/Nx;   % Length of cell in x 
dY = Ly/Ny;   % Length of cell in y
N = Nx*Ny;    % Number of cells
r_0 = Lx/2;   % Radius of workspace

%% Initialize
if isempty(init_flag)
   % Initial position (cell)
   idx_old = Nx*(Nx/2 + floor(eta_0(1)/dX)) + Ny/2 + ...
       floor(eta_0(2)/dY) + 1; 
   
   % Initial heading
   theta_old = rad2pipi(eta_0(3));
   
   % COLREGs scenario
   % Relative bearing
   phi = rad2pipi(atan2(p_TS(2) - eta_0(2),p_TS(1) - eta_0(1)) -...
       eta_0(3));
   % Relative course
   rel_course = rad2pipi(atan2(vel_TS(2),vel_TS(1)) - eta_0(3));
   % Calculate scenario
   if phi >= -pi/8 && phi <= pi/8       % [-22.5,22.5]
       if rel_course >= (pi - pi/8) || rel_course <= (-pi + pi/8)
           scenario = 'HO';             % [-180,-157.5] or % [157.5,180]
       elseif rel_course > (-pi + pi/8) && rel_course <= (-pi/2 + pi/8) 
           scenario = 'GW';             % (-157.5,-67.5]
       else
           scenario = 'SO';
       end
   elseif phi > pi/8 && phi <= pi/2     % (22.5,90]
       if rel_course >= (pi - pi/8) || rel_course <= (-pi + pi/8)
           scenario = 'HO';             % [-180,-157.5] or % [157.5,180]
       elseif rel_course > (-pi + pi/8) && rel_course <= (-pi/2 + pi/8)
           scenario = 'GW';             % (-157.5,-67.5]
       else
           scenario = 'SO';
       end
   elseif phi <= 5/8*pi && phi > pi/2   % (90,112.5]
       if rel_course > (-pi + pi/8) && rel_course <= (-pi/2 + pi/8)
           scenario = 'GW';             % (-157.5,-67.5]
       else
           scenario = 'SO';
       end
   elseif phi < -pi/8 && phi >= -pi/2   % [-90,-22.5)
       if rel_course >= (pi - pi/8) || rel_course <= (-pi + pi/8)
           scenario = 'HO';             % [-180,-157.5] or % [157.5,180]
       else
           scenario = 'SO';
       end
   else
       scenario = 'SO';
   end
   init_flag = 1;
end

%% Partitioning the workspace
[X,Y] = meshgrid((-r_0 + dX/2):dX:r_0,(-r_0 + dY/2):dY:r_0);
x = reshape(X,[N,1]);   % x-coordinates for center of cells 
y = reshape(Y,[N,1]);   % y-coordinates for center of cells 
A = zeros(N,N);         % Adjacency matrix
for i = 1:N
    for j = 1:N
        dist = sqrt((x(j) - x(i))^2 + (y(j) - y(i))^2);
        if (i ~= j) && (dist <= d_0) 
            A(i,j) = 1;
        end
    end
end

%%  Evolving the navigation function landscape
Z = zeros(1,N); % Initializing navigation function values
switch scenario
    case 'HO'
        for i = 1:N
            if sqrt(x(i)^2 + y(i)^2) > r_0 % Exterior of workspace
                Z(i) = 1;
            else
                % Virtual obstacles term
                term = 1;
                for j = 1:n_HO
                    term = term*((X(i) - (p_TS(1) + (r_TS + (2*j - 1)...
                    *r_HO)*cos(atan2(vel_TS(2),vel_TS(1)) + pi/8)))^2 ...
                    + (Y(i) - (p_TS(2) + (r_TS + (2*j - 1)*r_HO)*sin...
                    (atan2(vel_TS(2),vel_TS(1)) + pi/8)))^2 - (r_HO)^2);     
                end
                % Denominator
                denom = ((X(i)^2 + Y(i)^2)^kappa + ...
                    (r_0^2 - (X(i)^2 + Y(i)^2))*((X(i) - (p_TS(1)))^2 + ...
                    (Y(i) - (p_TS(2)))^2 - r_TS^2)*term);
                % Navigation function
                Z(i) = (X(i)^2 + Y(i)^2)/((abs(denom)^(1/kappa))...
                    *sign(denom));
                if Z(i) > 1
                    Z(i) = 1;
                end
            end
        end
    case 'GW'
        for i = 1:N
            if sqrt(x(i)^2 + y(i)^2) > r_0 % Exterior of workspace
                Z(i) = 1;
            else
                % Virtual obstacles term
                term = 1; 
                n_GW = ceil(Lx/(2*r_GW));
                for j = 1:n_GW
                    term = term*((X(i) - (p_TS(1) + (r_TS + (2*j - 1)...
                        *r_GW)*cos(atan2(vel_TS(2),vel_TS(1)))))^2 ...
                        + (Y(i) - (p_TS(2) + (r_TS + (2*j - 1)*r_GW)*sin...
                        (atan2(vel_TS(2),vel_TS(1)))))^2 - (r_GW)^2);     
                end
                % Denominator
                denom = ((X(i)^2 + Y(i)^2)^kappa + ...
                    (r_0^2 - (X(i)^2 + Y(i)^2))*((X(i) - (p_TS(1)))^2 + ...
                    (Y(i) - (p_TS(2)))^2 - r_TS^2)*term);
                % Navigation function
                Z(i) = (X(i)^2 + Y(i)^2)/((abs(denom)^(1/kappa))*...
                    sign(denom));
                if Z(i) > 1
                    Z(i) = 1;
                end
            end
        end
    otherwise
        for i = 1:N
            if sqrt(x(i)^2 + y(i)^2) > r_0 % Exterior of workspace
                Z(i) = 1;
            else
                % Denominator
                denom = ((X(i)^2 + Y(i)^2)^kappa + ...
                    (r_0^2 - (X(i)^2 + Y(i)^2))*((X(i) - (p_TS(1)))^2 + ...
                    (Y(i) - (p_TS(2)))^2 - r_TS^2));    
                % Navigation function
                Z(i) = (X(i)^2 + Y(i)^2)/((abs(denom)^(1/kappa))*...
                    sign(denom));
                if Z(i) > 1
                    Z(i) = 1;
                end
            end
        end
end

%% Plot
% figure(1)
% surf(Y,X,reshape(Z,[Nx,Nx])); 
% xlabel('y'); 
% ylabel('x');
% %  colorbar
% view(2)
% axis equal
% figure(2)
% surf(Y,X,reshape(Z,[Nx,Nx])); 
% xlabel('y'); 
% ylabel('x');
% %  colorbar
% view(3)
% axis([-Lx/2 Lx/2 -Lx/2 Lx/2 0 1])

%% Organize cell positions with potential field value
z_new = Z';
p = [x, y, z_new];

%%  Calculating new WP
idx_new = 0;
theta_new = 0;
[~,col] = find(A(idx_old,:));
min_val = Inf;
for jj = 1:length(col)
    theta_p = atan2(p(col(jj),2) - p(idx_old,2),...
        p(col(jj),1) - p(idx_old,1));
    theta_p_err = abs(rad2pipi(theta_p - theta_old));
    % Limit on heading change
    if theta_p_err >= pi/2
        val = Inf;
    else
        val = (1 + lambda_psi/pi*theta_p_err)*p(col(jj),3);  
    end
    % Steepest descent
    if val < min_val
        idx_new = col(jj);
        theta_new = theta_p;
        min_val = val;
    end
end

%% Update persistent
idx_old = idx_new;
theta_old = rad2pipi(theta_new);

%% Output
wp(:,count) = p(idx_new,1:2)';